@@ -1,5 +1,5 @@
 from com import Component, msgs
-from ui.Button import Button
+from ui.EventBox import EventBox
 from ui.ImageButton import ImageButton
 from ui.Label import Label
 from ui.itemview import ThumbableGridView
@@ -7,10 +7,11 @@ from ui.Slider import VSlider
 from ui.dialog import InputDialog
 from ui.dialog import OptionDialog
 from ui.dialog import ListDialog
-from ui.Pixmap import Pixmap
+from ui.Pixmap import Pixmap, text_extents
 from ui.Window import Window
+from ui.decorators import Gestures
 from StationItem import StationItem
-from FMScale import FMScale
+from FMScala import FMScala
 from FMRadio import *
 from SoundPipe import SoundPipe
 import platforms
@@ -64,6 +65,9 @@ _APP_MENU_EDITING = """
 """
 
 
+_FREQ_STEP = 50
+
+
 class FMRadioUI(Component, Window):
     """
     Player for FM radio stations.
@@ -92,48 +96,79 @@ class FMRadioUI(Component, Window):
 
         Component.__init__(self)
         Window.__init__(self, Window.TYPE_TOPLEVEL)
+        self.set_size(800, 423)
         self.set_title("FM Radio")
                                       
-        #self.set_visible(True)
         self.connect_closed(self.__on_close_window)
         self.connect_key_pressed(self.__on_key_pressed)
         
         
         # [Select] button
-        self.__btn_select = Button("")
+        self.__btn_select = EventBox()
         self.__btn_select.connect_clicked(self.__on_btn_select)
         self.add(self.__btn_select)
 
 
-        # FM scale
-        self.__fmscale = FMScale()
-        self.__fmscale.add_overlayer(self.__overlay_navigation_buttons)
-        self.__fmscale.set_range(87500, 108000)
-        #self.__fmscale.set_value(config.get_lastfreq())
-        self.__fmscale.set_stations(*self.__stations)
-        self.__fmscale.connect_frequency_changed(self.__on_tune)
-        self.__fmscale.connect_button_released(self.__on_click_scale)
-        #self.__fmscale.connect_scanning_activated(self.__on_scan)
-        self.add(self.__fmscale)
+        # FM scala
+        self.__fmscala = FMScala()
+        #self.__fmscala.add_overlayer(self.__overlay_glass)
+        #self.__fmscala.add_overlayer(self.__overlay_navigation_buttons)
+        self.__fmscala.set_range(87500, 108000)
+        #self.__fmscala.set_value(config.get_lastfreq())
+        self.__fmscala.set_stations(*self.__stations)
+        self.__fmscala.connect_frequency_changed(self.__on_tune)
+        self.__fmscala.connect_frequency_announced(self.__on_freq_announced)
+        #self.__fmscala.connect_button_released(self.__on_click_scala)
+        self.add(self.__fmscala)
       
         # label for RDS text
-        self.__lbl_rds = Label("", theme.font_fmradio_rds_label,
-                               theme.color_fmradio_rds_label)
+        self.__lbl_rds = Label("", theme.font_rds_text,
+                               theme.color_rds_text)
         self.__lbl_rds.set_alignment(Label.CENTERED)
         self.add(self.__lbl_rds)
 
+        # label for station name
+        self.__lbl_station = Label("", theme.font_rds_name,
+                                   theme.color_rds_name)
+        self.__lbl_station.set_alignment(Label.CENTERED)
+        self.add(self.__lbl_station)
+
+        # label for frequency
+        self.__lbl_freq = Label("", theme.font_frequency,
+                                theme.color_frequency)
+        self.__lbl_freq.set_alignment(Label.CENTERED)
+        self.add(self.__lbl_freq)
+
+        # navigation buttons
+        self.__btn_prev = ImageButton(theme.bt_prev,
+                                      theme.bt_prev_down)
+        self.add(self.__btn_prev)
+        gst = Gestures(self.__btn_prev)
+        gst.connect_tap(lambda *a:self.__skip_preset(-1))
+        gst.connect_hold(lambda *a:self.__scan_previous())
+
+        self.__btn_next = ImageButton(theme.bt_next,
+                                      theme.bt_next_down)
+        self.add(self.__btn_next)
+        gst = Gestures(self.__btn_next)
+        gst.connect_tap(lambda *a:self.__skip_preset(1))
+        gst.connect_hold(lambda *a:self.__scan_next())
+
 
         self.set_visible(True)
         gobject.idle_add(self.__init_app)
 
 
     def __init_app(self):
+        """
+        Initializes the application.
+        """
     
         if (self.__assert_headphones()):
             self.__radio_on()
-        
+            
         if (self.__radio):
-            self.__set_frequency(config.get_lastfreq())
+            self.__fmscala.set_value(config.get_lastfreq())
             self.__display_note("The Bluetooth module is responsible for\n" \
                                 "FM radio and enabled automatically.")
 
@@ -141,6 +176,9 @@ class FMRadioUI(Component, Window):
 
 
     def __display_note(self, msg):
+        """
+        Displays a banner note with the given message string.
+        """
           
         try:
             import hildon
@@ -172,17 +210,33 @@ class FMRadioUI(Component, Window):
         w, h = self.get_size()
         screen = self.get_screen()
         
-        screen.fill_area(x, y, w, h, theme.color_mb_background)
-        #screen.fill_area(4, 4, w - 8, 60, theme.color_fmradio_rds_bg)
+        screen.draw_pixbuf(theme.top, 0, 0)
+        self.__lbl_rds.set_geometry(10, 0, w - 20, 0)
+        self.__fmscala.set_geometry(81, 28, 638, 270)
+        self.__btn_prev.set_geometry(0, 28, 81, 270)
+        self.__btn_next.set_geometry(w - 81, 28, 81, 270)
         
-        self.__lbl_rds.set_geometry(0, 0, w, 80)
-        self.__fmscale.set_geometry(0, 80, w, h - 80 - 80)
-        self.__btn_select.set_geometry(0, h - 80, w, 80)
+        self.__render_bottom()
         
-        #screen.draw_pixbuf(theme.fmradio_rds, w - 200, 10)
         
-        #screen.draw_pixbuf(theme.fmradio_needle, (w - 24) / 2, h - 80 - 60)
+    def __render_bottom(self):
 
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        screen = self.get_screen()
+        
+        screen.draw_pixbuf(theme.bottom, 0, h - 125)
+                
+        self.__lbl_station.set_geometry(0, h - 100, w, 0)
+        self.__lbl_freq.set_geometry(0, h - 60, w, 0)
+        self.__btn_select.set_geometry(0, h - 100, w, 100)
+
+        #t_w, t_h = text_extents(self.__lbl_station.get_text(),
+        #                        theme.font_rds_name)
+        screen.draw_pixbuf(theme.bt_rdsname_arrow,
+                           w - 64, h - 64)
+
+        
 
     def __render_buffered(self):
     
@@ -219,6 +273,16 @@ class FMRadioUI(Component, Window):
                                  "rename": self.__on_menu_rename})
 
 
+    def __overlay_glass(self, obj, screen):
+        """
+        Overlays the glass pane.
+        """
+        
+        x, y = obj.get_screen_pos()
+        screen.draw_pixbuf(theme.scala_glass, 0, 0)
+    
+
+
     def __overlay_navigation_buttons(self, obj, screen):
         """
         Overlays the navigation buttons.
@@ -254,16 +318,16 @@ class FMRadioUI(Component, Window):
     def __on_key_pressed(self, key):
     
         if (key == "Left"):
-            self.__fmscale.step_down()
+            self.__fmscala.step_down()
             
         elif (key == "Right"):
-            self.__fmscale.step_up()
+            self.__fmscala.step_up()
 
             
 
     def __on_menu_add(self):
     
-        freq = self.__fmscale.get_value()
+        freq = self.__fmscala.get_value()
         name = ""
         
         # try to retrieve name from RDS
@@ -277,11 +341,11 @@ class FMRadioUI(Component, Window):
     
             ret = dlg.run()
             if (ret == 0):                
-                name = dlg.get_values()[0] or "%0.1f MHz" % (freq / 1000.0)
+                name = dlg.get_values()[0] or "%0.2f MHz" % (freq / 1000.0)
             
         if (name):
             self.__stations.append((freq, name))
-            self.__fmscale.set_stations(*self.__stations)
+            self.__fmscala.set_stations(*self.__stations)
 
             self.__update_menu()
             self.__render_buffered()
@@ -290,13 +354,13 @@ class FMRadioUI(Component, Window):
         
     def __on_menu_delete(self):
     
-        dlg = OptionDialog("Delete Station?")
-        dlg.add_option(None, "Delete")
+        dlg = OptionDialog("Remove Station?")
+        dlg.add_option(None, "Remove")
         dlg.add_option(None, "Cancel")
         
         ret = dlg.run()
         if (dlg.get_choice() == 0):
-            freq = self.__fmscale.get_value()
+            freq = self.__fmscala.get_value()
             new_stations = []
             for f, n in self.__stations:
                 if (freq == f):
@@ -306,7 +370,7 @@ class FMRadioUI(Component, Window):
             #end for
         
             self.__stations = new_stations
-            self.__fmscale.set_stations(*self.__stations)
+            self.__fmscala.set_stations(*self.__stations)
 
             self.__update_menu()
             self.__render_buffered()
@@ -325,7 +389,7 @@ class FMRadioUI(Component, Window):
         new_name = dlg.get_values()[0]
     
         new_stations = []
-        freq = self.__fmscale.get_value()
+        freq = self.__fmscala.get_value()
         for f, n in self.__stations:
             if (freq == f):
                 new_stations.append((f, new_name))
@@ -334,7 +398,7 @@ class FMRadioUI(Component, Window):
         #end for
         
         self.__stations = new_stations
-        self.__fmscale.set_stations(*self.__stations)
+        self.__fmscala.set_stations(*self.__stations)
 
         self.__update_menu()
         self.__stations_dirty = True
@@ -342,9 +406,11 @@ class FMRadioUI(Component, Window):
 
     def __on_menu_info(self):
     
-        self.__display_note("%s - version %s\n%s\n\n" \
-                            "Icon artwork by Andrew Zhilin" \
-                            % (values.NAME, values.VERSION, values.COPYRIGHT))
+        self.__display_note("%s - version %s\n\n" \
+                            "Written by Martin Grimme\n" \
+                            "Artwork by Patricia Montenegro\n" \
+                            "Icon by Andrew Zhilin" \
+                            % (values.NAME, values.VERSION))
         
 
     def __on_menu_output(self, choice):
@@ -358,15 +424,19 @@ class FMRadioUI(Component, Window):
             self.__sound_pipe.use_speaker(self.__use_speaker)
 
 
-    def __on_click_scale(self, px, py):
+    def __on_click_scala(self, px, py):
     
-        w, h = self.__fmscale.get_size()
+        w, h = self.__fmscala.get_size()
         if (px < 80):
             self.__skip_preset(-1)
         elif (px > w - 80):
             self.__skip_preset(1)
 
       
+    def __on_freq_announced(self, freq):
+    
+        self.__lbl_freq.set_text("%0.2f MHz" % (freq / 1000.0))
+      
         
     def __on_tune(self, v):
     
@@ -381,25 +451,9 @@ class FMRadioUI(Component, Window):
         
         self.__update_menu()
 
-        #if (self.__fmscale.get_value() != v):
-        #    self.__fmscale.set_value(v)
-
-
-    def __on_scan(self, direction):
-    
-        def f(freq):
-            self.__fmscale.set_value(freq)
-            while (gtk.events_pending()):
-                gtk.main_iteration(False)
-    
-        if (self.__radio):
-            if (direction < 0):
-                self.__radio.scan_previous(f)
-            else:
-                self.__radio.scan_next(f)
-            #self.__fmscale.set_value(self.__radio.get_frequency())
-        #end if
-        
+        #if (self.__fmscala.get_value() != v):
+        #    self.__fmscala.set_value(v)
+      
 
     def __on_btn_select(self):
     
@@ -419,7 +473,7 @@ class FMRadioUI(Component, Window):
         if (ret == 0):
             item = dlg.get_choice()
             freq = item.get_frequency()
-            self.__on_tune(freq)
+            self.__fmscala.set_value(freq)
         #end if
 
 
@@ -429,9 +483,10 @@ class FMRadioUI(Component, Window):
         string if there is no station set at the moment.
         """
     
-        freq = self.__fmscale.get_value()
+        freq = self.__fmscala.get_value()
         stations = [ n for f, n in self.__stations
                      if f == freq ]
+
         if (stations):
             return stations[0]
         else:
@@ -511,17 +566,17 @@ class FMRadioUI(Component, Window):
             # no presets available...
             return
 
-        current_freq = self.__fmscale.get_value()
+        current_freq = self.__fmscala.get_value()
         
         # find stations and distance to each station
         if (direction > 0):
             candidates = [ (freq, freq - current_freq)
                            for freq, name in self.__stations
-                           if freq > current_freq ]
+                           if freq > current_freq + _FREQ_STEP / 2 ]
         else:
             candidates = [ (freq, freq - current_freq)
                            for freq, name in self.__stations
-                           if freq < current_freq ]
+                           if freq < current_freq - _FREQ_STEP / 2 ]
 
         if (candidates):
             # sort by absolute distance
@@ -534,7 +589,7 @@ class FMRadioUI(Component, Window):
                 # take nearest
                 new_freq, dist = candidates[0]
                 
-            self.__set_frequency(new_freq)
+            self.__fmscala.set_value(new_freq)
 
         elif (not take_farthest):
             # try to wrap around
@@ -544,19 +599,50 @@ class FMRadioUI(Component, Window):
         #end if        
 
 
+    def __on_scan_progress(self, freq):
+    
+        if (freq % 200 == 0):
+            self.__fmscala.set_value(freq, False)
+            self.__lbl_freq.set_text("%0.2f MHz" % (freq / 1000.0))
+        
+        while (gtk.events_pending()):
+            gtk.main_iteration(False)
+
+
+    def __scan_previous(self):
+    
+        if (self.__radio):
+            self.__lbl_station.set_text("... Scanning ...")
+            current = self.__radio.get_frequency()
+            freq = self.__radio.scan_previous(self.__on_scan_progress)
+            if (freq != current):
+                self.__fmscala.set_value(freq)
+                self.__update_menu()
+        
+
+    def __scan_next(self):
+    
+        if (self.__radio):
+            self.__lbl_station.set_text("... Scanning ...")
+            current = self.__radio.get_frequency()
+            freq = self.__radio.scan_next(self.__on_scan_progress)
+            if (freq != current):
+                self.__fmscala.set_value(freq)
+                self.__update_menu()
+
+
     def __set_frequency(self, freq):
         """
-        Sets the radio and scale to the given frequency.
+        Sets the radio and scala to the given frequency.
         """
     
         if (self.__radio):
             self.__radio.set_frequency(freq)
             
-        self.__fmscale.set_value(freq)
-        self.__btn_select.set_text("%0.1f MHz" % (freq / 1000.0))
-        self.__btn_select.render()
+        self.__current_rds_ps = ""
+        name = self.__get_current_station_name()
+        self.__lbl_station.set_text(name)
         self.__lbl_rds.set_text("")
-        self.__btn_select.render()
 
 
     def __update_rds(self):
@@ -570,15 +656,13 @@ class FMRadioUI(Component, Window):
             rt = rt.strip() or ""
             
             if (ps):
-                self.__btn_select.set_text(ps)
-                self.__btn_select.render()
+                self.__lbl_station.set_text(ps)
+                self.__current_rds_ps = ""
                 
             if (rt and not self.__current_rds_rt.startswith(rt)):
                 self.__lbl_rds.set_text(rt)
-                self.__btn_select.render()
 
-            self.__current_rds_ps = ""
-            self.__current_rds_rt = rt
+                self.__current_rds_rt = rt
 
             return True
 
