@@ -20,8 +20,10 @@ class SoundPipe(object):
         self.__pipe_source = gst.element_factory_make("pulsesrc", "source")
         self.__pipe_queue = gst.element_factory_make("queue", "queue")
         self.__pipe_sink = gst.element_factory_make("pulsesink", "sink")
-        self.__pipe.add(self.__pipe_source, self.__pipe_queue, self.__pipe_sink)
-        gst.element_link_many(self.__pipe_source, self.__pipe_queue, self.__pipe_sink)
+        #self.__pipe.add(self.__pipe_source, self.__pipe_queue, self.__pipe_sink)
+        self.__pipe.add(self.__pipe_source, self.__pipe_sink)
+        #gst.element_link_many(self.__pipe_source, self.__pipe_queue, self.__pipe_sink)
+        gst.element_link_many(self.__pipe_source, self.__pipe_sink)
         
         
     def on(self):
